import os
import shutil
import sys
import tempfile
import urllib.request
import zipfile

import mpp.src.utils.constants as cst


def shell_exec_as_user():
    """
    Download ShellExecAsUser.dll for NSIS:
    https://nsis.sourceforge.io/mediawiki/images/c/c7/ShellExecAsUser.zip

    Returns:
        str: patht to the downloaded file
    """

    def get_zip_name(url):
        return url.split("/")[-1]

    def get_dll_name(url):
        url, _ = os.path.splitext(url)
        return f"{url}.dll"


    # URL and file names
    zipname = get_zip_name(cst.link_shellexecasuser)
    dllname = get_dll_name(zipname)
    dl_output = os.path.dirname(cst.path_dll_shellexecasuser)

    # Temporary download directory
    dir_temp = tempfile.gettempdir()
    dir_path = os.path.join(dir_temp, "tmp_shellexecasuser")
    dl_path = os.path.join(dir_path, zipname)

    # Get ShellExecAsUser.zip file
    if os.path.exists(dl_path):
        print(f"Using cached file from {dl_path}")
    else:
        os.mkdir(dir_path)
        __download(cst.link_shellexecasuser, dl_path)

    # Extract ShellExecAsUSer.dll
    with zipfile.ZipFile(dl_path) as zf:
        zf.extract(dllname, dl_output)

    return cst.path_dll_shellexecasuser


def __download(url_src, path_dst):
    """
    Download a file from an url to a file

    Args:
        url_src (str): url of the file
        path_dst (str): path to the output file
    """

    try:
        print("Downloading...", end=" ")
        with urllib.request.urlopen(url_src) as response:
            with open(path_dst, "wb") as output:
                shutil.copyfileobj(response, output)
        print("Done")
    except urllib.error.URLError:
        sys.exit("No internet connection.")




def question(message, default="", required=False):
    """
    Ask a question to the user

    Args:
        message (str): message to display
        default (str): the default value
        required (bool): if True, loop to get an answer

    Returns:
        str: user's value
    """

    # Do while
    while True:
        # Adapt the message to add the default answer
        if default != "":
            message += f" [{default}] "
        elif not message.endswith(" "):
            message += " "

        # Ask the question
        answer = input(message)
        if answer == "":
            answer = default

        # Verify if it should be asked again
        if answer != "":
            break
        else:
            if required is False:
                break

    return answer



import json

from mpp.src.utils import constants as cst


def write_installer(mpp_config):
    """
    Writes the PyInstaller spec file and the NSIS file

    Args:
        mpp_config (dict): project parameters
    """

    # Rewrite the specs file
    with open(f"installer/installer.spec", "w") as f:
        f.write(cst.pattern_spec % mpp_config)
    # Rewrite the nsis file
    with open(f"installer/installer.nsi", "w") as f:
        f.write(cst.pattern_nsis % mpp_config)



def write_mpp_config(mpp_config):
    """
    Writes the project's configuration file

    Args:
        mpp_config (dict): project parameters
    """

    # Write the config file
    with open(".mpp_config", "w") as f:
        json.dump(mpp_config, f, indent=4)



import json
import os
import shutil
import sys
import textwrap

from mpp.src.utils import ask, constants as cst, files


def config(args=None):
    """
    Show or edit configuration parameters

    Args:
        args (argparse args): parameters from parser.parse_args()
    """

    if not any([args.list, args.parameters]):
        print("This command needs parameters.")
        print("Use `mpp config --help` to show the help.")
        sys.exit()

    # Get project config file
    if not os.path.exists(".mpp_config"):
        sys.exit("Please setup your environment by using the 'setup' command")
    with open(".mpp_config") as f:
        mpp_config = json.load(f)

    # If there is no parameter
    if args.list:
        __show_config(mpp_config)
        sys.exit()

    # If there are parameters
    for param in args.parameters:
        if param not in mpp_config:
            valid = [f"'{x}'" for x in mpp_config.keys()]
            valid = ", ".join(valid)
            sys.exit(f"Invalid parameter: '{param}' (choose from {valid})")

    new_config = __process_parameters(args, mpp_config)
    if new_config:
        mpp_config.update(**new_config)
        files.write_mpp_config(mpp_config)
        files.write_installer(mpp_config)


def __show_config(mpp_config):
    """
    Shows the parameters from the configurtion file

    Args:
        mpp_config (dict): project parameters
    """

    values = [f" -→ {k} = {v}" for k, v in mpp_config.items()]
    print(*values, sep="\n")


def __process_parameters(args, mpp_config):
    """
    Processes the given parameters

    Args:
        args (argparse args): parameters from parser.parse_args()
        mpp_config (dict): project parameters

    Returns:
        dict: user's answers
    """

    # Process each parameter
    answers = dict()
    if "name" in args.parameters:
        answers["name"] = ask.question(
            "What is your project name?",
            default=mpp_config["name"]
        )
    if "author" in args.parameters:
        answers["author"] = ask.question(
            "What is your author name?",
            default=mpp_config["author"]
        )
    if "version" in args.parameters:
        answers["version"] = ask.question(
            "What is the new version?",
            default=mpp_config["version"]
        )
    if "console" in args.parameters:
        answers["console"] = ask.question(
            "Do you want to display the console (y/n)?",
            default="y" if mpp_config["console"] else "n"
        )
        answers["console"] = answers["console"].lower() == "y"
    if "hidden-imports" in args.parameters:
        answers["hidden-imports"] = __process_hidden_imports(mpp_config)

    # Validate modifications
    print("")
    is_ok = ask.question(
        "Are you sure of your modifications (y/n)?",
        required=True
    ).lower() == "y"

    if not is_ok:
        answers.clear()

    return answers


def __process_hidden_imports(mpp_config):
    """
    Processes the `hidden-imports` parameter

    Args:
        mpp_config (dict): project parameters

    Returns:
        list: user's hidden-imports
    """

    imports = mpp_config["hidden-imports"][:]
    help_msg = textwrap.dedent("""\
    Use `-<package>` to remove a package or `+<package>` to add it.
    Use `list` to display the current imports.
    Use `clear` to remove all the packages.
    Use `help` to show this message.
    Use `q` to exit.""")

    print("List of current hidden imports:")
    print(f"[{', '.join(imports)}]")
    print("")
    print(help_msg)

    while True:
        answer = input("> ")

        # Verify input
        if len(answer.split()) > 1:
            print("White spaces are not allowed.")
            continue

        # Exit
        if answer == "q":
            break
        # Show help
        elif answer == "help":
            print(help_msg)
        # Remove all
        elif answer == "clear":
            imports.clear()
        # Display list of imports
        elif answer == "list":
            print(f"[{', '.join(imports)}]")
        # Remove one package
        elif answer.startswith("-"):
            try:
                imports.remove(answer[1:])
            except ValueError:
                print(f"`{answer[1:]}` is not part of the hidden imports.")
        # Add one package
        elif answer.startswith("+"):
            imports.append(answer[1:])
        # Something else
        else:
            print(f"`{answer}` is not a valid entry")

    return sorted(set(imports))



import json
import os
import shutil

from mpp.src.utils import ask, constants as cst, files


def setup(args=None):
    """
    Asks information to the user and setup the environment

    Args:
        args (argparse args): parameters from parser.parse_args()
    """

    # Get information to forecast user's mpp_config
    current_dir = os.path.basename(os.getcwd())
    username = os.path.basename(os.path.expanduser("~"))

    # Ask questions
    mpp_config = dict()
    mpp_config["name"] = ask.question("What is your project name?", current_dir, required=True)
    mpp_config["author"] = ask.question("What is your author name?", username, required=True)
    mpp_config["version"] = "0.0.0"
    mpp_config["console"] = ask.question("Do you want to display the console (y/n)?", "y")
    mpp_config["console"] = mpp_config["console"].lower() == "y"
    mpp_config["icon"] = "resources/images/icon.ico"
    mpp_config["hidden-imports"] = list()

    # Create folders
    os.makedirs("installer", exist_ok=True)
    os.makedirs("resources/images", exist_ok=True)
    os.makedirs("src", exist_ok=True)

    # Add icon
    if not os.path.exists(mpp_config["icon"]):
        shutil.copy(cst.path_ico_default, mpp_config["icon"])
    # Write configuration file
    files.write_mpp_config(mpp_config)
    # Write main file
    if not os.path.exists("main.py"):
        with open("main.py", "w") as f:
            f.write(cst.pattern_main_py % mpp_config)

    files.write_installer(mpp_config)

    print("")
    print(f"The project version is {mpp_config['version']}")
    print(f"The project's icon is here: {mpp_config['icon']}.")
    print("The `main.py` file can now be edited.")
    print("")
    print("Use `mpp --help` to display all possible commands")
    print("Use `mpp <command> -h` to display the help for a command.")
    print("Use `mpp config --list` to show your project settings.")



import os

pp = os.path.dirname
dir_home = pp(pp(pp(__file__)))

path_ico_default = os.path.join(dir_home, "resources", "default.ico")
path_dll_shellexecasuser = os.path.join("installer", "ShellExecAsUser.dll")
link_shellexecasuser = "https://nsis.sourceforge.io/mediawiki/images/c/c7/ShellExecAsUser.zip"

with open(os.path.join(dir_home, "resources", "main.py.pattern")) as f:
    pattern_main_py = f.read()

with open(os.path.join(dir_home, "resources", "spec.pattern")) as f:
    pattern_spec = f.read()

with open(os.path.join(dir_home, "resources", "nsis.pattern")) as f:
    pattern_nsis = f.read()



#!/usr/bin/env python

import argparse
import sys

from mpp.src.commands import setup, config, freeze, installer


__version__ = "0.0.4"


def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    parser.add_argument("--version", action="store_true", help="show version and exit")

    setup_parser = subparsers.add_parser(
        "setup",
        description="Ask some questions to setup the project",
        help="ask some questions to setup the project"
    )
    setup_parser.set_defaults(func=setup)

    config_parser = subparsers.add_parser(
        "config",
        description="Show project parameters and edit them",
        help="show project parameters and edit them"
    )
    config_parser.add_argument("parameters", metavar="parameter", nargs="*", help="parameters to edit")
    config_parser.add_argument("--list", action="store_true", help="show project parameters")
    config_parser.set_defaults(func=config)

    freeze_parser = subparsers.add_parser(
        "freeze",
        description="Create an executable using PyInstaller",
        help="create an executable using PyInstaller"
    )
    freeze_parser.set_defaults(func=freeze)

    installer_parser = subparsers.add_parser(
        "installer",
        description="Create an installer with NSIS",
        help="create an installer with NSIS"
    )
    installer_parser.set_defaults(func=installer)

    args = parser.parse_args()

    if args.version == True:
        print(f"My Python Project {__version__}")
        sys.exit(0)
    elif not hasattr(args, "func"):
        parser.print_help()
        sys.exit(0)

    args.func(args)



from setuptools import find_packages, setup

import mpp.main


with open("README.md") as f:
    long_description = f.read()

setup(
    name="mpp",
    version=mpp.main.__version__,
    author="deplanty",
    description="My Python Project: a command-line tool to create a Python project",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/deplanty/mpp",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "mpp = mpp.main:main",
        ]
    },
    classifiers=[
        "Natural Language :: English",
        "Operating System :: Microsoft :: Windows",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    python_requires=">=3.7"
)



import json
import os
import shutil
import sys

from mpp.src.commands import freeze
from mpp.src.utils import ask, download, constants as cst


def installer(args=None):
    """
    Creates an installer with the created executable with `freeze`

    Args:
        args (argparse args): parameters from parser.parse_args()
    """

    # Load parameters file
    with open(".mpp_config") as f:
        mpp_config = json.load(f)

    # Freeze command executed
    if not os.path.isdir(f"target/{mpp_config['name']}"):
        print("It seems that the `freeze` command wasn't executed.")
        answer = ask.question("Do you want to do it now (y/n)?", "y")
        print("")
        if answer == "y":
            freeze()
        else:
            sys.exit()

    # Makensis exists
    if not shutil.which("makensis"):
        print("It seems that NSIS is not installed or set in the PATH.")
        print("It can be found at this address: https://nsis.sourceforge.io/Download")
        sys.exit()

    # ShellExecAsUser exists
    if not os.path.exists(cst.path_dll_shellexecasuser):
        print("NSIS needs \"ShellExecAsUser\" in order to create the installer.")
        answer = ask.question("Do you want to download it (y/n)?", "y")
        print("")
        if answer == "y":
            download.shell_exec_as_user()
            print("")
        else:
            print("It can be found at this address: https://nsis.sourceforge.io/ShellExecAsUser_plug-in")
            sys.exit()

    # Execute the makensis
    print("~$ makensis installer.nsi")
    os.chdir("installer")
    os.system("makensis installer.nsi")
    os.chdir("..")
    print("")
    print(f"Installer can be found here: target/{mpp_config['name']}_setup.exe")



import json
import json
import os
import shutil
import sys

from mpp.src.utils import ask


def freeze(args=None):
    """
    Freezes the project to create an executable

    Args:
        args (argparse args): parameters from parser.parse_args()
    """

    with open(".mpp_config") as f:
        mpp_config = json.load(f)

    # PyInstaller exists
    if not shutil.which("pyinstaller"):
        print("It seems that PyInstaller is not installed.")
        print("Please, consider using `pip install PyInstaller`.")
        print(f"Current pip is {shutil.which('pip')}.")
        answer = ask.question("Do you want to install it now (y/n)?", "y")
        print("")
        if answer == "y":
            print("~$ pip install PyInstaller")
            os.system("pip install PyInstaller")
            print("")
        else:
            sys.exit()

    # Execute PyInstaller
    print("~$ pyinstaller installer.spec")
    os.chdir("installer")
    os.system("pyinstaller installer.spec")
    os.chdir("..")
    print("")
    print(f"Executable can be found here: target/{mpp_config['name']}/{mpp_config['name']}.exe")
